"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const react = require("react");
const ApiContext = react.createContext(null);
const dummyApi = {
  getSearch: () => "",
  replaceState: () => {
  },
  pushState: () => {
  },
  registerListener: () => {
  }
};
const withBatch = (api) => {
  let isBatching = false;
  let lastActionDetails = null;
  return {
    ...api,
    getSearch: () => {
      if (isBatching && lastActionDetails) {
        return lastActionDetails.search;
      }
      return api.getSearch();
    },
    replaceState: (search, state) => {
      if (isBatching) {
        lastActionDetails = {
          search,
          state,
          type: "replaceState"
        };
      } else {
        api.replaceState(search, state);
      }
    },
    pushState: (search, state) => {
      if (isBatching) {
        lastActionDetails = {
          search,
          state,
          type: "pushState"
        };
      } else {
        api.pushState(search, state);
      }
    },
    batch: (fn) => {
      isBatching = true;
      fn();
      if (lastActionDetails) {
        api[lastActionDetails.type](lastActionDetails.search, lastActionDetails.state);
      }
      isBatching = false;
    }
  };
};
const defaultApi = () => {
  return {
    getSearch: () => window.location.search,
    replaceState: (params, state) => window.history.replaceState(state, "", params),
    pushState: (params, state) => window.history.pushState(state, "", params),
    registerListener: (listener) => {
      const originalPushState = window.history.pushState;
      window.history.pushState = (state, ...args) => {
        originalPushState.apply(window.history, [state, ...args]);
        listener(state);
      };
      const orginalReplaceState = window.history.replaceState;
      window.history.replaceState = (state, ...args) => {
        orginalReplaceState.apply(window.history, [state, ...args]);
        listener(state);
      };
      window.addEventListener("popstate", () => {
        listener();
      });
    }
  };
};
exports.ApiContext = ApiContext;
exports.defaultApi = defaultApi;
exports.dummyApi = dummyApi;
exports.withBatch = withBatch;
//# sourceMappingURL=api.cjs.map
