import { useMemo, useRef } from "react";
import { isEqual } from "lodash-es";
const getRenderingType = () => {
  const isBrowser = typeof window != "undefined" && window.document;
  return isBrowser ? "client" : "server";
};
const getValue = (value) => {
  if (typeof value === "function") {
    return value();
  }
  return value;
};
const isClient = getRenderingType() === "client";
const isServer = getRenderingType() === "server";
const paramsTransitioning = "__ParamsTransitioning";
const isParamsTransition = (state) => state != null && typeof state === "object" && paramsTransitioning in state;
const useSmartValue = (value) => {
  const internalValue = getValue(value);
  const ref = useRef(void 0);
  if (ref.current === void 0) {
    ref.current = internalValue;
  }
  if (isEqual(ref.current, internalValue)) {
    return ref.current;
  } else {
    ref.current = internalValue;
    return internalValue;
  }
};
const useMemoOptions = (options, options2, options3) => {
  const smartOptions1 = useSmartValue(options);
  const smartOptions2 = useSmartValue(options2);
  const smartOptions3 = useSmartValue(options3);
  return useMemo(() => mergeOptions(smartOptions1, smartOptions2, smartOptions3), [smartOptions1, smartOptions2, smartOptions3]);
};
const mergeOptions = (...options) => {
  return Object.assign({}, ...options);
};
const camelCaseToKebab = (str) => str.replace(/[A-Z]+(?![a-z])|[A-Z]/g, ($, ofs) => (ofs ? "-" : "") + $.toLowerCase());
export {
  camelCaseToKebab,
  getRenderingType,
  getValue,
  isClient,
  isParamsTransition,
  isServer,
  mergeOptions,
  paramsTransitioning,
  useMemoOptions,
  useSmartValue
};
//# sourceMappingURL=utils.js.map
